@@ -1,11 +1,12 @@
-#define NETDISSECT_REWORKED
+/* \summary: Solaris DLT_IPNET printer */
+
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 
-#include <tcpdump-stdinc.h>
+#include <netdissect-stdinc.h>
 
-#include "interface.h"
+#include "netdissect.h"
 
 typedef struct ipnet_hdr {
 	uint8_t		iph_version;
@@ -55,7 +56,7 @@ ipnet_hdr_print(netdissect_options *ndo, const u_char *bp, u_int length)
 static void
 ipnet_print(netdissect_options *ndo, const u_char *p, u_int length, u_int caplen)
 {
-	ipnet_hdr_t *hdr;
+	const ipnet_hdr_t *hdr;
 
 	if (caplen < sizeof(ipnet_hdr_t)) {
 		ND_PRINT((ndo, "[|ipnet]"));
@@ -67,7 +68,7 @@ ipnet_print(netdissect_options *ndo, const u_char *p, u_int length, u_int caplen
 
 	length -= sizeof(ipnet_hdr_t);
 	caplen -= sizeof(ipnet_hdr_t);
-	hdr = (ipnet_hdr_t *)p;
+	hdr = (const ipnet_hdr_t *)p;
 	p += sizeof(ipnet_hdr_t);
 
 	switch (hdr->iph_family) {
@@ -82,7 +83,7 @@ ipnet_print(netdissect_options *ndo, const u_char *p, u_int length, u_int caplen
 
 	default:
 		if (!ndo->ndo_eflag)
-			ipnet_hdr_print(ndo, (u_char *)hdr,
+			ipnet_hdr_print(ndo, (const u_char *)hdr,
 					length + sizeof(ipnet_hdr_t));
 
 		if (!ndo->ndo_suppress_default_print)
