@@ -12,8 +12,17 @@ then
 	printf '    %-35s: TEST SKIPPED (no Makefile)\n' $test_name
 elif grep '^CC = .*gcc' ../Makefile >/dev/null
 then
-	./TESTonce $test_name isis-seg-fault-1.pcap isis-seg-fault-1-v.out '-v'
-	[ $? -eq 0 ] || exitcode=1
+	passed=`cat .passed`
+	failed=`cat .failed`
+	if ./TESTonce $test_name isis-seg-fault-1.pcap isis-seg-fault-1-v.out '-v'
+	then
+		passed=`expr $passed + 1`
+		echo $passed >.passed
+	else
+		failed=`expr $failed + 1`
+		echo $failed >.failed
+		exitcode=1
+	fi
 else
 	printf '    %-35s: TEST SKIPPED (compiler is not GCC)\n' $test_name
 fi
